<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 *  Class for working with Gentics Content Connector
 */
class RepositoryApi extends CApplicationComponent
{
    const OBJ_TYPE_FOLDER = 10002;
    const OBJ_TYPE_PAGE = 10007;
    const OBJ_TYPE_BINARY = 10008;

    /**
     * @var string api url
     */
    public $url;

    public $failover_urls = array();

    public $contentRepositorySearch;
    /**
     * @var string binary content service path
     */
    public $binaryContent;

    public $navigation;

    public $activePath;

    public $autosuggest;

    public $search;

    public $contentExternalHost;

    /**
     * @var array parameters which will be added to every request to gentics content connector
     */
    public $additionalRequestParameters = array();

    /**
     * @var array Attributes, which will be added to every request as "&attribute=something"
     */
    public $additionalAttributes = array();

    /**
     * @var array Sorting-Rules, that can be added to spezific requests
     */
    public $additionalSortingRules = array();


    /**
     * Get api url which used to retrieve CMS objects
     *
     * @return string
     */
    protected function getContentRepositorySearchUrl()
    {
        return $this->getUrl() . $this->contentRepositorySearch;
    }

    /**
     * Get api url which used to retrieve static objects(images, js, css, etc.)
     *
     * @return string
     */
    protected function getBinaryContentUrl()
    {
        return $this->getUrl() . $this->binaryContent;
    }

    /**
     * Get api url which used to retrieve navigation data
     *
     * @return string
     */
    protected function getNavigationUrl()
    {
        return $this->getUrl() . $this->navigation;
    }

    /**
     * Get api url which used to retrieve active path data
     *
     * @return string
     */
    protected function getActivePathUrl()
    {
        return $this->getUrl() . $this->activePath;
    }

    /**
     * Get api url which used to retrieve autosuggest data
     *
     * @return string
     */
    protected function getAutosuggestUrl()
    {
        return $this->getUrl() . $this->autosuggest;
    }

    /**
     * Get api url which used to retrieve search data
     *
     * @return string
     */
    protected function getSearchUrl()
    {
        return $this->getUrl() . $this->search;
    }

    /**
     * Set base options
     */
    public function __construct()
    {
        if (isset($_GET['lang']) && is_string($_GET['lang'])) {
            Yii::app()->language = $_GET['lang'];
        }
    }

    protected function getUrl(){
        // The main API was failed and we check if there
        //is detected auxiliary API server in the cache
        if(($failover_URL = Yii::app()->cache->get('failover_URL'))!==false){
            $this->failover_urls[] = $this->url;
            $this->url = $failover_URL;
        }
        return $this->url;
    }

    /**
     * Do almost the same as getCmsObject. But in case relative path $path not found in the CMS it adds host address
     * in front of $path and try second time
     *
     * @param $path
     * @param array $attributes
     * @return array|bool
     */
    public function getCmsObjectCanBeExternal($path, $attributes = array())
    {
        $cmsObject = $this->getCmsObject($path, $attributes);
        if (!$cmsObject && !preg_match('/.+\.(php|html)$/', $path)) {
            $externalPath = (!empty($this->contentExternalHost) ? $this->contentExternalHost : Yii::app()->request->getHostInfo()) . $path;
            return $this->getCmsObject($externalPath, $attributes);
        }

        return $cmsObject;
    }

    /**
     * Get CMS content last modification time
     *
     * @param string $path path of node
     *
     * @return bool
     */
    public function getLastModificationTime($path)
    {
        $cacheId = 'updateTMSTMP_'.md5($path);
        $updateTMSTMP = false;
        if(!($updateTMSTMP = Yii::app()->cache->get($cacheId)) || (isset($_GET['refresh']) && $_GET['refresh']=='true')){
            $attributes = $this->getContentAttributes($path, array('updatetimestamp'));
            if($attributes){
                $updateTMSTMP = $attributes['updatetimestamp'];
                Yii::app()->cache->set($cacheId, $updateTMSTMP, Yii::app()->getModule('contentSource')->cacheTime);
            }
        }
        return $updateTMSTMP;
    }

    /**
     * This method is used for retrieving only content attributes from Gentics Content Connector
     *
     * @param string $path       content path
     * @param array  $attributes array of personalization attributes
     *
     * @return array
     */
    public function getContentAttributes($path, $attributes)
    {
        $cmsObject = $this->getCmsObject($path, array_filter($attributes, function ($el) {
            return $el != 'content';
        }));
        return $cmsObject['attributes'];
    }

    /**
     * Get CMS content object
     * If object is binary, make additional request to get binarycontent and put it to 'content' field
     *
     * @param string $path       content path
     * @param array  $attributes requested content attributes
     *
     * @return array|bool
     */
    public function getCmsObject($path, $attributes = array())
    {
        $params = array(
            'filter' => "object.url==\"$path\"",
            'type' => 'php',
        );

        $params = $this->addAdditionalParameters($params, 'contentRepositorySearch');
        $response = $this->_request($this->getContentRepositorySearchUrl(), $params, $attributes);
        $cmsObject = self::_prepareResponse($response);
        if (!$cmsObject) {
            return false;
        }
        if (in_array('content', $attributes) && $cmsObject['obj_type'] == self::OBJ_TYPE_BINARY) {
            $cmsObject['attributes']['content'] = $this->getBinaryContent($path);
        }

        return $cmsObject;
    }

    /**
     * Download binary content
     *
     * @param string $path content path
     *
     * @return string binary content
     */
    public function getBinaryContent($path)
    {
        $params = array(
            'filter' => "object.url==\"$path\"",
        );
        $params = $this->addAdditionalParameters($params, 'binaryContent');

        $response = $this->_request($this->getBinaryContentUrl(), $params);
        return $response;
    }

    /**
     * Adds parameters to certain requests
     *
     * @param array  $params array of preset parameters
     * @param string $type   type of request from
     *
     * @return array
     */
    public function addAdditionalParameters($params, $type)
    {
        if (!empty($this->additionalRequestParameters[$type])) {
            foreach ($this->additionalRequestParameters[$type] as $name => $val) {
                $params[$name] = !empty($params[$name]) ? $params[$name] . ' AND ' . $val : $params[$name];
            }
        }
        return $params;
    }

    /**
     * Make request to API
     *
     * @param string $url        url
     * @param array  $params     parameters
     * @param array  $attributes array of Gentics connector attributes which need to be select
     *
     * @throws Exception
     * @return string raw response
     */
    private function _request($url, $params, $attributes = array())
    {
        $attr = array();
        array_walk($attributes, function ($item) use (&$attr) {
            $attr[] = "attributes=$item";
        });

        $url = $url . '?' . http_build_query($params);
        foreach($this->additionalSortingRules as $addRuleK => $addRuleV){
            if( strpos($url,$addRuleK)){
                $url .= '&sorting='.$addRuleV; 
            }
        }
        $url .=  '&' . implode('&', $attr). '&' . implode('&attributes=', $this->additionalAttributes);
        $url = rtrim($url, '&');
        Yii::trace($url, 'Api requests(start)');
        $start = microtime(true);
        $ctx=stream_context_create(array('http'=>
                array(
                    'timeout' => 1
                )
        ));
        /*  do not remove the @ sign below, it is required for exception handling */
        @$content = file_get_contents($url, false, $ctx);
        $failover_attempt = 0;
        while(($content == false || strpos($content,'DatasourceException')!==false) && ($failover_attempt < count($this->failover_urls))){
            $this->url = $this->failover_urls[$failover_attempt];
            $exploded_url = explode('://',$url);
            $exploded_url = explode('/',$exploded_url[1]);
            $exploded_url[0] = $this->url;
            $url = implode('/', $exploded_url);
            @$content = file_get_contents($url, false, $ctx);
            if($content !== false){
                Yii::app()->cache->set('failover_URL', $this->url, Yii::app()->getModule('contentSource')->cacheTime);
                break;
            }
            $failover_attempt++;
        }
        if ($content == false) {
            throw new Exception('Repository API error: could not complete API request.');
        }
        Yii::trace($url, 'Api requests(end). Time s.: ' . round(microtime(true) - $start, 4));

        return $content;
    }

    /**
     * Requesting for content object
     *
     * @param array $params     url params
     * @param array $attributes requested attributes
     *
     * @return string
     */
    public function request($params, $attributes = array())
    {
        return $this->_request($this->getContentRepositorySearchUrl(), $params, $attributes);
    }

    /**
     * Requesting for navigation object
     *
     * @param array $params     url params
     * @param array $attributes requested attributes
     *
     * @return string
     */
    public function requestNavigation($params, $attributes = array())
    {
        return $this->_request($this->getNavigationUrl(), $params, $attributes);
    }

    /**
     * Requesting for active path object
     *
     * @param array $params     url params
     * @param array $attributes requested attributes
     *
     * @return string
     */
    public function requestActivePath($params, $attributes = array())
    {
        return $this->_request($this->getActivePathUrl(), $params, $attributes);
    }

    /**
     * Requesting for autosuggest data
     *
     * @param array $params     url params
     * @param array $attributes requested attributes
     *
     * @return string
     */
    public function requestAutosuggest($params, $attributes = array())
    {
        return $this->_request($this->getAutosuggestUrl(), $params, $attributes);
    }

    /**
     * Requesting for search data
     *
     * @param array $params     url params
     * @param array $attributes requested attributes
     *
     * @return string
     */
    public function requestSearch($params, $attributes = array())
    {
        return $this->_request($this->getSearchUrl(), $params, $attributes);
    }

    /**
     * Prepare API response. Convert response to useful form.
     *
     * @param array $response prepared response
     *
     * @return array|bool prepared object.
     *
     */
    private static function _prepareResponse($response)
    {
        $responseData = unserialize($response);
        if (@$responseData['status'] == 'ok') {
            unset($responseData['status']);
            if (count($responseData) > 0) {
                return array_shift($responseData);
            }
            return false;
        } else {
            Yii::log(print_r($responseData, true), CLogger::LEVEL_TRACE);
        }
        return false;
    }
}
