<?php
/**
 * Compress and cache used JS and CSS files.
 * Needs jsmin and cssmin
 *
 * Ties into the 1.0.4 and up Yii CClientScript functions
 *
 * 0.9.0 Now using CssMin code.google.com/p/cssmin/ for PHP 5.3.x compatibility
 *
 * Now checking and excluding remote files automatically
 *
 * @author Maxximus <maxximus007@gmail.com>
 * @author Alexander Makarov <sam@rmcreative.ru>
 * @author Kir <>
 *
 * @link http://www.yiiframework.com/
 * @copyright Copyright &copy; 2008-2011
 * @license htp://www.yiiframework.com/license/
 * @version 0.9.0
 *
 */

class ExtendedClientScript extends CClientScript
{

    public $disableRegisterFiles = false;
    /**
     * Compress all Javascript files with JSMin. JSMin must be installed as an extension in $jssminPath.
     * github.com/rgrove/jsmin-php/
     */
    public $compressCombinedJs = false;
    /**
     * Compress all CSS files with CssMin. CssMin must be installed as an extension in $cssMinPath.
     * Specific browserhacks will be removed, so don't add them in to be compressed CSS files.
     * code.google.com/p/cssmin/
     */
    public $compressCombinedCss = false;
    /**
     * DEPRECATED/LEGACY
     * Combine all JS and CSS files into one. Be careful with relative paths in CSS.
     */
    public $combineFiles = false;
    /**
     * Combine all non-remote JS files into one.
     */
    public $combineJs = false;
    /**
     * Combine all non-remote CSS files into one. Be careful with relative paths in CSS.
     */
    public $combineCss = false;
    /**
     * Exclude certain files from inclusion. array('/path/to/excluded/file') Useful for fixed base
     * and incidental additional JS.
     */
    public $excludeFiles = array();
    /**
     * Path where the combined/compressed file will be stored. Will use coreScriptUrl if not defined
     */
    public $filePath;

    /**
     * @var string domain for splitting js/css files across domains
     */
    public $splitDomainName;

    /**
     * Relative Url where the combined/compressed file can be found
     */
    public $fileUrl;
    /**
     * Path where files can be found
     */
    public $basePath;
    /**
     * Used for garbage collection. If not accessed during that period: remove.
     */
    public $ttlDays = 1;
    /**
     * prefix for the combined/compressed files
     */
    public $prefix = 'c_';
    /**
     * path to JsMin
     */
    public $jsMinPath = 'ext.ExtendedClientScript.jsmin.*';
    /**
     * path to CssMin
     */
    public $cssMinPath = 'ext.ExtendedClientScript.cssmin.*';
    /**
     * CssMin filter options. Default values according cssMin doc.
     */
    public $cssMinFilters = array
    (
        'ImportImports' => false,
        'RemoveComments' => true,
        'RemoveEmptyRulesets' => true,
        'RemoveEmptyAtBlocks' => true,
        'ConvertLevel3AtKeyframes' => false,
        'ConvertLevel3Properties' => false,
        'Variables' => true,
        'RemoveLastDelarationSemiColon' => true
    );
    /**
     * CssMin plugin options. Maximum compression and conversion.
     */
    public $cssMinPlugins = array
    (
        'Variables' => true,
        'ConvertFontWeight' => true,
        'ConvertHslColors' => true,
        'ConvertRgbColors' => true,
        'ConvertNamedColors' => true,
        'CompressColorValues' => true,
        'CompressUnitValues' => true,
        'CompressExpressionValues' => true,
    );

    private $_timestamps = array();
    private $_gccFile = array();

    /**
     * Registers a script package that is listed in {@link packages}.
     * @param string $name the name of the script package.
     * @return CClientScript the CClientScript object itself (to support method chaining, available since version 1.1.5).
     * @see renderCoreScript
     */
    public function registerCoreScript($name, $force = false)
    {
        if ($this->disableRegisterFiles && !$force) {
            return $this;
        }
        return parent::registerCoreScript($name);
    }

    public function registerCssFile($file, $media = '', $timestamp = null, $combine = true, $gccFile = false, $force = false)
    {
        if ($this->disableRegisterFiles && !$force) {
            return $this;
        }

        $this->_gccFile[$file] = $gccFile;
        if ($combine == false) {
            $this->excludeFiles[] = $file;
        } elseif ($timestamp !== null)
            $this->_timestamps['css'][$file] = $timestamp;
        return parent::registerCssFile($file, $media);
    }

    public function registerScriptFile($file, $position = self::POS_HEAD, $timestamp = null, $combine = true, $gccFile = false, $force = false)
    {
        if ($this->disableRegisterFiles && !$force) {
            return $this;
        }

        $this->_gccFile[$file] = $gccFile;
        if ($combine == false) {
            $this->excludeFiles[] = $file;
        } elseif ($timestamp !== null)
            $this->_timestamps['js'][$file] = $timestamp;
        return parent::registerScriptFile($file);
    }

    /**
     * Will combine/compress JS and CSS if wanted/needed, and will continue with original
     * renderHead afterwards
     *
     * @param <type> $output
     */
    public function renderHead(&$output)
    {
        if ($this->combineFiles) {
            $this->combineJs = $this->combineCss = true;
        }

        $this->processJs(parent::POS_HEAD);
        $this->processCss();

        parent::renderHead($output);
    }

    /**
     * Will combine/compress JS if wanted/needed, and will continue with original
     * renderBodyEnd afterwards
     *
     * @param <type> $output
     */
    public function renderBodyBegin(&$output)
    {
        $this->processJs(parent::POS_BEGIN);
        parent::renderBodyBegin($output);
    }

    /**
     * Will combine/compress JS if wanted/needed, and will continue with original
     * renderBodyEnd afterwards
     *
     * @param <type> $output
     */
    public function renderBodyEnd(&$output)
    {
        $this->processJs(parent::POS_END);
        parent::renderBodyEnd($output);
    }


    /**
     * @param <type> $output
     * @param <type> $pos
     */
    private function processJs($pos)
    {
        if ($this->combineJs) {
            if (isset($this->scriptFiles[$pos]) && count($this->scriptFiles[$pos]) !== 0) {
                $jsFiles = array();
                foreach ($this->scriptFiles[$pos] as $key => $file) {
                    if (!in_array($file, $this->excludeFiles)) {
                        $jsFiles[$key] = $file;
                    } else {
                        //add domain to url if exists
                        unset($this->scriptFiles[$pos][$key]);
                        $this->scriptFiles[$pos][$this->addSplitDomain($file)] = $this->addSplitDomain($file);
                    }
                }
                $this->combineAndCompress('js', $jsFiles);
            }
        }
    }

    private function processCss()
    {
        if ($this->combineCss) {
            if (count($this->cssFiles) !== 0) {
                $cssFiles = array();
                foreach ($this->cssFiles as $url => $media) {
                    if (!in_array($url, $this->excludeFiles)) {
                        $cssFiles[$media][$url] = $url;
                    } else {
                        //add domain to url if exists
                        unset($this->cssFiles[$url]);
                        $this->cssFiles[$this->addSplitDomain($url)] = $media;
                    }
                }
                foreach ($cssFiles as $media => $url) {
                    $this->combineAndCompress('css', $url);
                }
            }
        }
    }

    /**
     * Performs the actual combining and compressing
     *
     * @param <type> $type
     * @param <type> $urls
     * @param <type> $pos
     */
    private function combineAndCompress($type, $urls)
    {
        $this->fileUrl or $this->fileUrl = $this->getCoreScriptUrl();
        $this->basePath or $this->basePath = realpath($_SERVER['DOCUMENT_ROOT']);
        $this->filePath or $this->filePath = $this->basePath . $this->fileUrl;

        $optionsHash = ($type == 'js') ?
            md5($this->basePath . $this->compressCombinedJs . $this->ttlDays . $this->prefix) :
            md5($this->basePath . $this->compressCombinedCss . $this->ttlDays . $this->prefix . serialize($this->cssMinFilters) . serialize($this->cssMinPlugins));
        $combineHash = md5(implode('', $urls));
        $fileName = $this->prefix . md5($combineHash . $optionsHash) . ".$type";
        $timestampHash = !empty($this->_timestamps[$type]) ? md5(implode('', $this->_timestamps[$type])) : '';

        $timestampChanged = Yii::app()->cache->get($fileName) != $timestampHash;
        $fileNotExists = (file_exists($this->filePath . '/' . $fileName)) ? false : true;
        if ($fileNotExists || $timestampChanged) {
            $this->garbageCollect($type);
            $combinedFile = '';
            foreach ($urls as $file) {
                $combinedFile .= $this->_getFileContent($file);
            }
            if ($type == 'js' && $this->compressCombinedJs) {
                $combinedFile = $this->minifyJs($combinedFile);
            } elseif ($type == 'css' && $this->compressCombinedCss) {
                $combinedFile = $this->minifyCss($combinedFile);
            }
            file_put_contents($this->filePath . '/' . $fileName, $combinedFile);
            Yii::app()->cache->set($fileName, $timestampHash);
        }
        foreach ($urls as $url)
            $this->scriptMap[basename($url)] = $this->addSplitDomain($this->fileUrl . '/' . $fileName);

        $this->remapScripts();
    }

    private function garbageCollect($type)
    {
        $files = CFileHelper::findFiles($this->filePath, array('fileTypes' => array($type), 'level' => 0));
        foreach ($files as $file) {
            if (strpos($file, $this->prefix) !== false && $this->fileTTL($file)) {
                unlink($file);
            }
        }
    }

    /**
     * Add domain name to url
     *
     * @param $url
     */
    private function addSplitDomain($url)
    {
        if ($this->splitDomainName && !$this->isRemoteFile($url)) {
            return (Yii::app()->request->getIsSecureConnection() ? 'https' : 'http') . '://' . $this->splitDomainName . $url;
        } else {
            return $url;
        }
    }

    /**
     * This function gets content of file depends of file type:
     *
     * If file is remote - download it
     * If file from Gentics Content Connector - request it via API
     * If file from webroot - read it as file
     *
     * @param string $file name of file
     * @return string
     */
    private function _getFileContent($file)
    {
        if ($this->isRemoteFile($file)) {
            return file_get_contents($file);
        } elseif (isset($this->_gccFile[$file]) && $this->_gccFile[$file]) {
            $content = Yii::app()->getModule('contentSource')->getContent($file);
            if($content === false){
                Yii::trace('CMS static file not exists '. $file, 'compressing');
                return '';    
            } else {
                return file_get_contents($content->getFile());
            }
            return file_get_contents($content->getFile());
        } else {
            return file_get_contents(Yii::getPathOfAlias('webroot') . '/' . $file);
        }
    }

    /**
     * See if file is ready for deletion
     *
     * @param <type> $file
     * @return bool
     */
    private function fileTTL($file)
    {
        if (!file_exists($file)) {
            return false;
        }
        $ttl = $this->ttlDays * 60 * 60 * 24;
        return ((fileatime($file) + $ttl) < time()) ? true : false;
    }

    /**
     * Minify javascript with JSMin
     *
     * @param <type> $js
     */
    private function minifyJs($js)
    {
        Yii::import($this->jsMinPath);
        return JSMin::minify($js);
    }

    /**
     * Minify css with cssmin
     *
     * @param <type> $css
     */
    private function minifyCss($css)
    {
        Yii::import($this->jsMinPath);
        Yii::import($this->cssMinPath);
        return cssmin::minify($css, $this->cssMinFilters, $this->cssMinPlugins);
    }

    /**
     * See if file is on remote server
     *
     * @param <type> $file
     */
    private function isRemoteFile($file)
    {
        return (strpos($file, 'http://') === 0 || strpos($file, 'https://') === 0) ? true : false;
    }

    private $_baseUrl;

    public function getCoreScriptUrl()
    {
        if ($this->_baseUrl !== null) {
            return $this->_baseUrl;
        } else {
            return $this->_baseUrl = Yii::app()->getAssetManager()->publish(YII_PATH . '/web/js/source', true);
        }
    }


}
