<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Widget for view comments for current model
 */
class CommentsListWidget extends CommentsBaseWidget
{
    /**
     * @var bool If threaded commenting allowed (only 1 level)
     */
    public $oneLevelReply = false;

    /**
     * @var string Widget options hash, to get this options in controller
     */
    public $hash;

    /**
     * @var CommentsFormWidget comment form which is used together with this CommentsListWidget
     */
    public $formWidget;

    /**
     * @var int comments per page
     */
    public $commentsPerPage = 5;

    /**
     * @var int how many pagination pages numbers will be displayed at the same time
     */
    public $paginationPagesCount = 5;

    /**
     * Make widgets options hash key and store options in session
     *
     * @return void
     */
    public function init()
    {
        parent::init();
        $assetsUrl = Yii::app()->getModule('comments')->getAssetsUrl();
        Yii::app()->clientScript->registerCssFile($assetsUrl . '/css/pagination.css');
        Yii::app()->clientScript->registerScriptFile($assetsUrl . '/js/initCommentsReply.js');

        $options = array(
            'contentId' => $this->contentId
        );
        $this->hash = md5(serialize($options) . Yii::app()->getModule('comments')->hashSalt);
        if (Yii::app()->cache->get($this->hash) != $options) {
            Yii::app()->cache->set($this->hash, $options);
        }
    }

    /**
     * Find all comments by contentId. Preload child comments.
     *
     * @return void
     */
    public function run()
    {
        $comments = $this->_loadComments();
        $this->render(
            'CommentsListWidget',
            array(
                'comments' => $comments['comments'],
                'commentsTotal' => $comments['total'],
                'commentsCountWithChildren' => $comments['countWithChildren']
            )
        );
    }

    /**
     * Load comments from cache.
     *
     * @return Like
     */
    private function _loadComments()
    {
        $currentPage = $this->getCurrentPage();
        $manageCommentsAccess = Yii::app()->user->checkAccess('manageComments');
        $isEmailModerator = $manageCommentsAccess ? false : CommentsModule::isEmailModerator(Yii::app()->user->email);
        //Complex cacheId. Merge dependency on: contentId, if user is administrator and if user is moderator.
        //For all administrator  users cache is generic.
        //For each moderator user cache is specific.
        //For all other users cache is generic.
        $cacheId = md5(
            implode(
                ',',
                array(
                    'parentComments',
                    $this->contentId,
                    $currentPage,
                    $manageCommentsAccess,
                    $isEmailModerator ? Yii::app()->user->email : null
                )
            )
        );
        //load comments hierarchy
        if (($comments = Yii::app()->cache->get($cacheId)) === false) {
            $criteria = new CDbCriteria();
            $criteria->addCondition('t.content_id=:content_id');
            if (!$manageCommentsAccess) {
                if ($isEmailModerator) {
                    $criteria->addCondition('(t.moderator_email=:moderator_email OR t.status=:status)');
                    $criteria->params[':moderator_email'] = Yii::app()->user->email;
                } else {
                    $criteria->addCondition('t.status=:status');
                }
                $criteria->params[':status'] = Comment::APPROVED;
            }
            $criteria->order = 't.createtime DESC';
            $criteria->params[':content_id'] = (string)$this->contentId;

            //apply pagination
            $commentsTotal = Comment::model()->isParent()->count($criteria);
            $paginator = new CPagination($commentsTotal);
            $paginator->pageSize = $this->commentsPerPage;
            $paginator->currentPage = $currentPage;
            $paginator->applyLimit($criteria);
            $comments = Comment::model()->isParent()->findAll($criteria);

            $childComments = array();
            if ($this->oneLevelReply) {
                $pIds = array();
                foreach ($comments as $pc) {
                    $pIds[] = $pc->id;
                }
                $criteria->addInCondition('parent_id', $pIds);
                $criteria->order = 't.createtime ASC';
                $criteria->limit = -1;
                $criteria->offset = -1;
                $childComments = Comment::model()->findAll($criteria);
                $childrenByParents = array();
                foreach ($childComments as $cc) {
                    $childrenByParents[$cc->parent_id][] = $cc;
                }
                foreach ($comments as $pc) {
                    $pc->childs = !empty($childrenByParents[$pc->id]) ? $childrenByParents[$pc->id] : array();
                }
            }

            //save to cache
            if (Yii::app()->db->getDriverName() == 'mysql') {
                //use MySql specific function for dependency
                $dependency = new CDbCacheDependency("SELECT group_concat(id, ' - ' ,status) FROM " . Comment::model()->tableName() . " WHERE content_id=:content_id ");
                $dependency->params['content_id'] = $this->contentId;
            } else {
                //use general sql expression for all db. more memory usage
                $dependency = new CExpressionDependency(
                    'Yii::app()->db->createCommand()->select("id, status")->from(Comment::model()->tableName())->where("content_id=:content_id", array("content_id" => ' . $this->contentId . '))->queryAll()'
                );
            }
            $comments = array(
                'comments' => $comments,
                'countWithChildren' => count($comments) + count($childComments),
                'total' => $commentsTotal,
            );
            Yii::app()->cache->set($cacheId, $comments, Yii::app()->getModule('comments')->cacheTime, $dependency);
        }

        return $comments;
    }

    /**
     * Returns current page
     *
     * @return integer
     */
    public function getCurrentPage()
    {
        return Yii::app()->request->getParam("clw{$this->contentId}", 0);
    }

    /**
     * Check if reply possibility allowed for current comments list
     *
     * @return bool flag that shows if user allowed reply for comment or not
     */
    public function canReply()
    {
        return $this->oneLevelReply && $this->formWidget && ($this->formWidget->allowAnonymous || !Yii::app()->user->isGuest);
    }

    /**
     * Returns current page
     *
     * @param integer $page page number
     *
     * @return string
     */
    public function createPaginationUrl($page)
    {
        $url = Yii::app()->request->getUrl();
        $name = "clw{$this->contentId}";
        $newParam = "clw{$this->contentId}=$page";
        if (($oldValue = Yii::app()->request->getParam($name)) !== null) {
            $url = str_replace("$name=$oldValue", $newParam, $url);
        } else {
            if (strpos($url, '?') !== false) {
                $url .= '&' . $newParam;
            } else {
                $url .= '?' . $newParam;
            }
        }
        $url .= '#list_' . $this->contentId;

        return $url;
    }

    /**
     * Returns comment attributes
     *
     * @param array $comments array of comments
     *
     * @return array
     */
    public function getSafeCommentAttributes($comments)
    {
        $commentsAttributes = array();
        foreach ($comments as $parent) {
            $parentData = array(
                'id' => $parent->id,
                'subject' => $parent->subject
            );
            foreach ($parent->childs as $child) {
                $parentData['childs'][] = array(
                    'id' => $child->id,
                    'subject' => $child->subject
                );
            }
            $commentsAttributes[] = $parentData;
        }
        return $commentsAttributes;
    }

    /**
     * Get comment author avatar image
     *
     * @param array $comment a comment
     *
     * @return String
     */
    public function getAvatar($comment)
    {
        return Gravatar::getImage($comment->email, Yii::app()->request->isSecureConnection ? 'https' : 'http');
    }
}

?>
