<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * This class represents content taken from Gentics Content Connector.
 * The main purpose is to store content attributes in cache
 * and in case cache expired or crashed reload this attributes.
 */
class Content
{
    private $_path;

    private $_attributes;

    private $_personalisationAttributes;

    private $_cachePrefix = 'content_';

    private $_source;

    /**
     * Content active path
     *
     * @return string
     */
    public function getPath()
    {
        return $this->_path;
    }

    /**
     * Content file location
     *
     * @return string
     */
    public function getFile()
    {
        return $this->_source->contentFile($this->getPath());
    }

    /**
     * Save content attributes to cache
     *
     * @param array $attributes content attributes
     *
     * @return void
     */
    public function setAttributes($attributes)
    {
        $this->_attributes = $attributes;
        if ($this->_source->usePersonalisation) {
            Yii::app()->cache->set($this->_cachePrefix . $this->_path, $attributes, Yii::app()->getModule('contentSource')->cacheTime);
        }
    }

    /**
     * Get content attributes from cache if $this->_source->usePersonalization== true
     * If cache is empty but personalisation enabled, request api for content attributes
     *
     * @throws Exception
     * @return mixed
     */
    public function getAttributes()
    {
        if ($this->_attributes === null && $this->_source->usePersonalisation) {
            $this->_attributes = Yii::app()->cache->get($this->_cachePrefix . $this->_path);
            if ($this->_attributes === false) {
                $attributes = $this->_source->getRepositoryApi()->getContentAttributes($this->_path, $this->_source->getRequestedAttributes());
                if ($attributes !== false) {
                    $this->setAttributes($attributes);
                } else {
                    $this->_attributes = false;
                }
            }
        }

        return $this->_attributes;
    }

    /**
     * Get content personalisation attributes
     *
     * @return mixed
     */
    public function getPersonalisationAttributes()
    {
        if ($this->_personalisationAttributes === null) {
            $this->_personalisationAttributes = array();
            if ($attributes = $this->getAttributes()) {
                foreach ($this->_source->personalisationFields as $pf) {
                    if (isset($attributes[$pf]) && is_array($attributes[$pf])) {
                        $this->_personalisationAttributes = array_merge($this->_personalisationAttributes, $attributes[$pf]);
                    }
                }
            }
        }
        return $this->_personalisationAttributes;
    }

    /**
     * Constructor
     *
     * @param ContentSource $source     content source. Used for getting current personalisation configuration
     * @param string        $path       content active path
     * @param array         $attributes content attributes
     *
     * @return \Content
     */
    public function __construct(ContentSource $source, $path, $attributes = null)
    {
        $this->_source = $source;
        $this->_path = $path;
        if ($attributes !== null) {
            $this->setAttributes($attributes);
        }
    }

    /**
     * Check if user has access to this content
     *
     * @param object $user user
     *
     * @return bool
     */
    public function hasAccess($user)
    {
        if ($this->_source->usePersonalisation) {
            $hasAccess = Yii::app()->getModule('personalisation')->rule->checkAccess($user->id, $this->getPersonalisationAttributes());
            Yii::trace("Check personalisation access. Result: [$hasAccess].".print_r($this->getPersonalisationAttributes(), true),'personalisation');
            return $hasAccess;
        }
        return true;
    }

}
