<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Class for loading content from Dynamic Content Renderer
 */
class DynamicContentSource extends ContentSource
{
    /**
     * @var string folder into which content will be cached
     */
    public $cacheFolder;

    private $_modTimeCachePrefix = '__modTime__';

    /**
     * Init Gentics Connector API
     */
    public function init()
    {

    }

    /**
     * Return array of attributes which will be requested from API
     *
     * @return array array of attributes
     */
    public function getRequestedAttributes()
    {
        $mainAttributes = array(
            'contentid',
            'name',
            'filename',
            'updatetimestamp',
            'languagecode',
            'mimetype',
            'content'
        );
        if ($this->usePersonalisation) {
            return array_merge($mainAttributes, $this->personalisationFields);
        }
        return $mainAttributes;
    }

    /**
     * Get content absolute path.
     * Checking if we have newest version on content,
     * if yes return, else download and store
     *
     * @param string $path content path
     *
     * @return \Content|mixed object contained file path, false if error
     */
    public function getContent($path)
    {
        if ($this->_haveNewestVersion($path)) {
            return $this->_takeFromFileSystem($path);
        } else {
            return $this->_takeAndSaveFromApi($path);
        }
    }

    public function getStartPage($folderPath, $locale)
    {
        $localizedAttributeName = Yii::app()->getModule('contentSource')->localizedAttributeName. $locale;

        $folderObject = $this->getRepositoryApi()->getContentAttributes($folderPath, array($localizedAttributeName));
        if($folderObject){
            return $folderObject[$localizedAttributeName];
        }
        return false;
    }

    /**
     * This method takes content from API and save locally
     *
     * @param string $path content path
     *
     * @return bool|string
     */
    private function _takeAndSaveFromApi($path)
    {
        if ($cmsObject = $this->getRepositoryApi()->getCmsObjectCanBeExternal($path, $this->getRequestedAttributes())) {
            $this->_createFile($path, $cmsObject['attributes']['content'], $cmsObject['attributes']['updatetimestamp']);
            $content = new Content($this, $path);
            unset($cmsObject['attributes']['content']);
            $content->setAttributes($cmsObject['attributes']);
            return $content;
        }

        return false;
    }

    /**
     * Checking if we have latest version of file
     *
     * @param string $path content path
     *
     * @return bool
     */
    private function _haveNewestVersion($path)
    {
        $contentFile = $this->contentFile($path);
        if (!file_exists($contentFile)) {
            return false;
        }

        if ($lastModificationTime = $this->getRepositoryApi()->getLastModificationTime($path)) {
            return
                Yii::app()->cache->get($this->_modTimeCachePrefix . $path) == $lastModificationTime ||
                filemtime($this->contentFile($path)) == $lastModificationTime;
        }

        return false;
    }

    /**
     * Take existed file from filesystem
     *
     * @param string $path content path
     *
     * @return bool|string
     */
    private function _takeFromFileSystem($path)
    {
        $contentFile = $this->contentFile($path);
        if ($this->fileAccessible($contentFile, $this->cacheFolder)) {
            $content = new Content($this, $path);
            return $content;
        } else {
            return false;
        }
    }


    /**
     * Create file,and save its last modification date
     *
     * @param string $path    string relative path
     * @param string $content string file content
     * @param int    $time    mixed last modification time
     *
     * @throws Exception
     * @return string string absolute path
     *
     */
    private function _createFile($path, $content, $time)
    {
        $contentFile = $this->contentFile($path);
        if (file_exists($contentFile) && !$this->fileAccessible($contentFile, $this->cacheFolder)) {
            return false;
        }

        $dirPath = pathinfo($contentFile, PATHINFO_DIRNAME);
        if (!file_exists($dirPath)) {
            if (!@mkdir($dirPath, 0777, true)) {
                throw new Exception("Can't create dir $dirPath");
            }
        }
        if (!file_put_contents($contentFile, $content)) {
            throw new Exception("Can't create file $contentFile");
        }
        if (!touch($contentFile, $time)) {
            throw new Exception("Can't touch() file $contentFile");
        }
        if (!chmod($contentFile, 0666)) {
            throw new Exception("Can't chmod() file $contentFile");
        }

        if (!$this->fileAccessible($contentFile, $this->cacheFolder)) {
            unlink($contentFile);
            return false;
        }

        Yii::app()->cache->set($this->_modTimeCachePrefix . $path, $time, Yii::app()->getModule('contentSource')->cacheTime);

        return $contentFile;
    }

    /**
     * Get content file location
     *
     * @param string $path content path
     *
     * @return string
     */
    public function contentFile($path)
    {
        return $this->cacheFolder . $path;
    }


}
