(function($) {
	
	/**
	 * @class Toolbar
	 */
	$.fg.Toolbar = {
			
			/**
			 * Toolbar area DOM element
			 * @type {object}
			 */
			object: null,
			/**
			 * Loader activity counter<br />
			 * When somebody turns the loader activity to true, the counter increments<br />
			 * This used for hiding loader only after the last attempt to hide loader when counter will equal to zero
			 * @type {number}
			 */
			loaderActivityCounter: 0,
			/**
			 * The collection of toolbar buttons
			 * @type {array}
			 */
			buttons: [],
		
		/**
		 * Initialize the toolbar
		 * @return {void}
		 */
		init: function()
		{
			this.buttons = [
				{
					name: "undo",
					title: t( "toolbar.action-undo" ),
					info: "Ctrl + Z",
					options: {
						icons: {
							primary: "ui-icon-arrowrefresh-1-w"
						},
						disabled: true
					}
				},
				{
					name: "redo",
					title: t( "toolbar.action-redo" ),
					info: "Ctrl + Y",
					options: {
						icons: {
							primary: "ui-icon-arrowrefresh-1-e"
						},
						disabled: true
					}
				},
				"|",
				{
					name: "save",
					title: t( "toolbar.action-save" ),
					options: {
						icons: {
							primary: "ui-icon-disk"
						}
					}
				},
				{
					name: "save_as_template",
					title: t( "toolbar.action-save_as_template" ),
					options: {
						icons: {
							primary: "ui-icon-disk"
						}
					}
				},
				{
					name: "edit",
					title: t( "toolbar.action-edit" ),
					options: {
						icons: {
							primary: "ui-icon-pencil"
						}
					}
				}
			];
			
			this.object = $( "#toolbar" );
			this.object.text( "" );
			
			// add toolbar buttons
			var ul = $( "<ul></ul>" ).addClass( "buttons" );
			$.each(this.buttons, function(k,v){
				var li = $( "<li></li>" );
				if(v === "|") {
					li.addClass( "separator" );
				} else {
					li
						.attr( "id", "toolbar-button-"+v.name )
						.addClass( v.name )
						.append(
							$( "<a></a>" )
								.attr( "href", "#" )
								.attr( "title", v.info || "" )
								.text( v.title )
								.click( $.fg.Toolbar[v.name+"Handler"] )
								.button( v.options || {} )
						);
				}
				ul.append( li );
			});
			this.object.append( ul );
			
			// add form edit button
			this.object.append(
				$( "<a></a>", {
					href: "#",
					text: t( "toolbar.action-editform" ),
					click: this.editformHandler
				} )
					.addClass( "editform" )
					.button( {
						icons: {
							primary: "ui-icon-pencil"
						}
					} )
			);
			
			// add language selector
			var currentLanguage = $.fg.Lang.getCurrentLanguage();
			var select = $( "<select>" ).addClass( "language" );
			$.each( $.fg.Config.languages, function(k, v) {
				select.append( $( "<option>", {value: k, text: v, selected: currentLanguage == k} ) );
			} );
			select.change( this.languageHandler );
			this.object.append( select );
			
			// add global process loader
			this.object.append(
				$( "<span></span>", {
					text: t( "toolbar.loader-text" )
				} )
					.addClass( "loader" )
			);
			
			// bind undo/redo buttons
			$(document).bind( "fg.undoredo.change", function(e) {
				$.fg.Toolbar.getButton( "undo" ).button( $.fg.UndoRedo.canUndo()? "enable" : "disable" );
				$.fg.Toolbar.getButton( "redo" ).button( $.fg.UndoRedo.canRedo()? "enable" : "disable" );
			} );
		},
		
		/**
		 * Get button from toolbar
		 * @param {string} name The name of the button
		 * @return {object} button DOM element
		 */
		getButton: function(name)
		{
			return this.object.find( "li."+ name +" a" );
		},
		
		/**
		 * Show toolbar loader (spinner with text "loading...")
		 * @param {boolean} activity show loader - {true} or hide - {false}
		 * @return {void}
		 */
		setLoaderActivity: function(activity)
		{
			if(activity) {
				this.loaderActivityCounter++;
			} else {
				this.loaderActivityCounter--;
			}
			if(this.loaderActivityCounter < 0) {
				this.loaderActivityCounter = 0;
			}
			this.object.find( "span.loader" ).toggleClass( "active", this.loaderActivityCounter > 0 );
		},
		
		/**
		 * Handler of language selection event<br />
		 * Setting current language and rerender some editor parts
		 * @param {event} e
		 * @return {void}
		 */
		languageHandler: function(e)
		{
			$.fg.Editor.setUserState( "lang", $(this).val() );
			$.fg.Lang.init(function()
			{
				// refresh
				$.fg.Toolbar.init();
				$.fg.Library.init();
				var element = $.fg.ElementEditor.getCurrentElement();
				if(element) {
					$.fg.ElementEditor.edit( element.id );
				}
			});
		},
		
		/**
		 * Handler of click on button "Save"<br />
		 * Saving current form for publishing
		 * @param {event} e
		 * @return {void}
		 */
		saveHandler: function(e)
		{
			e.preventDefault();
			$.fg.Editor.save();
		},
		
		/**
		 * Handler of click on button "Save as template"<br />
		 * Saving current form in the templates
		 * @param {event} e
		 * @return {void}
		 */
		save_as_templateHandler: function(e)
		{
			e.preventDefault();
			$.fg.Editor.saveTemplate();
		},
		
		/**
		 * Handler of click on button "Edit"<br />
		 * Shows library
		 * @param {event} e
		 * @return {void}
		 */
		editHandler: function(e)
		{
			e.preventDefault();
			
			$.fg.Library.toggle();
			//$.fg.ElementEditor.edit(0);
		},
		
		/**
		 * Handler of click on button "Edit form"<br />
		 * Shows properties of the form
		 * @param {event} e
		 * @return {void}
		 */
		editformHandler: function(e)
		{
			e.preventDefault();
			$.fg.ElementEditor.edit(0);
		},
		
		/**
		 * Handler of click on button "Undo"<br />
		 * Roll back changes
		 * @param {event} e
		 * @return {void}
		 */
		undoHandler: function(e)
		{
			e.preventDefault();
			$.fg.UndoRedo.undo();
		},
		
		/**
		 * Handler of click on button "Redo"<br />
		 * Restore changes after undo
		 * @param {event} e
		 * @return {void}
		 */
		redoHandler: function(e)
		{
			e.preventDefault();
			$.fg.UndoRedo.redo();
		}
		
	};
	
})(jQuery);