<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 *  Class for working with Gentics Content Connector
 */
class NavigationTree extends CComponent
{

    /**
     * id of the root folder for current widget
     * @var $startfolderId
     */
    public $startfolderId = 0;

    /**
     * id of the root folder for current widget
     * @var $activeElement
     */
    public $activeElement = 0;

    /**
     * array that contains categories tree
     * @var $data
     */
    public $data = array();

    /**
     * array that contains Ids for active categories
     * @var $activePath
     */
    public $activePath = array();

    /**
     * @var $lang string containg current languagecode
     */
    public $lang;

    /**
     * @var bool $usePersonalisation defines if personalisation should be performed
     */
    public $usePersonalisation = true;

    public function getTreeCacheTime()
    {
        return Yii::app()->getModule('navigation')->cacheTime;
    }

    public function getLatestUpdateTimestampCacheTime()
    {
        return Yii::app()->getModule('navigation')->timestampCacheTime;
    }

    /**
     * @var array $additionalParams defines additional Parameters to request
     */
    public $additionalParams = array();

    /**
     * @var string $sortby defines the field for sorting 
     */
    public $sortby = "sortorder:asc";

    /**
     * Construction that will load nodes tree for given startfolderId and activePath for given activeElement
     *
     * @param integer $startfolderId id of folder that must be treated as root
     * @param int     $activeElement id
     *
     * @return \NavigationTree
     */
    public function __construct($startfolderId = 0, $activeElement = 0, $additionalParams = array(), $sortby = "sortorder:asc")
    {
        if ($startfolderId != '' && (int)$startfolderId > 0) {
            $this->lang = substr(Yii::app()->language, 0, 2);

            $this->startfolderId = $startfolderId;
            $this->activeElement = $activeElement;
            $this->additionalParams = $additionalParams;
            $this->sortby = $sortby;
            $this->_loadData();
            $this->_loadActivePath();
        }
    }

    /**
     * Filter tree by some schema
     *
     * @param integer $user_id            Id of the user to filter tree for
     * @param bool    $usePersonalisation defines if personalisation of received content should be performed
     *
     * @return array
     */
    public function cleanUp($user_id, $usePersonalisation = true)
    {
        //get filter fields from settings
        $filters = $this->getFiltersList();

        $this->usePersonalisation = $usePersonalisation;

        //load tree data into local array
        $data = $this->data;
        // apply filter here
        if ($filters && $data) {
            $data = $this->_unsetNodes($filters, $user_id, $data);
        }

        return $data;
    }

    /**
     * Private method that will unset not allowed data
     *
     * @param array $filters contains keys that will be used as filters
     * @param array $user_id contains user permissions
     * @param array $data    points to the array with nodes tree
     *
     * @return array|null
     */
    private function _unsetNodes($filters, $user_id, $data)
    {
        /*  check permissions for current node */
        $persAttributes = array();
        foreach ($filters as $filter) {
            if (!empty($data['attributes'][$filter])) {
                $persAttributes = array_merge($persAttributes, $data['attributes'][$filter]);
            }
        }

        //check if page has allowed startpageurl and exlude the branch if it does not.
        if ($data['contentid'] != $this->startfolderId) {
            $excludeURLs = Yii::app()->getModule('navigation')->excludeURLs;

            /*  if isset startpage url for current language - use it, otherwise check if isset default startpage url and use it. In other cases set startpageurl to empty string */
            $startpageUrl = isset($data['attributes']['startpageurl_' . $this->lang]) ? $data['attributes']['startpageurl_' . $this->lang] : (isset($data['attributes']['startpageurl']) ? $data['attributes']['startpageurl'] : '');

            /*  if startpageurl is one those that should be excluded or empty - exclude it */
            if (empty($startpageUrl) || in_array($startpageUrl, $excludeURLs)) {
                return null;
            }
        }

        // Call function that will check permissions
        if ($this->usePersonalisation && !Yii::app()->getModule('personalisation')->rule->checkAccess($user_id, $persAttributes)) {
            /* if user does not have permissions - unset whole node */
            return null;
        } else if (isset($data['children'])) {
            /* check every child for permission */
            foreach ($data['children'] as $kChild => $child) {
                $data['children'][$kChild] = $this->_unsetNodes($filters, $user_id, $child);
                if (is_null($data['children'][$kChild])) {
                    unset($data['children'][$kChild]);
                }
            }
            if (count($data['children']) == 0) {
                unset($data['children']);
            }
        }
        return $data;
    }

    /**
     * Filter tree by some schema
     *
     * @return array|bool
     */
    public function getFiltersList()
    {
        if (Yii::app()->getModule('contentSource')->contentSource->usePersonalisation) {
            $filters = Yii::app()->getModule('contentSource')->contentSource->personalisationFields;
            return $filters;
        }

        return false;
    }

    /**
     * Load required tree data from cache. If not exists request Gentics Content Connector API for a new tree.
     *
     * @return array
     */
    private function _loadData()
    {
        $sysLangs = Yii::app()->getModule('language')->languages;
        if (empty($sysLangs[$this->lang])) {
            $sysLangs[$this->lang] = $this->lang;
        }

        $requestFields = array(
            'updatetimestamp',
            'node_id',
            'sortorder',
            'navhidden',
            'name',
            'startpageurl'
        );

        foreach ($sysLangs as $lang) {
            $requestFields[] = 'name_' . $lang;
            $requestFields[] = 'startpageurl_' . $lang;
        }

        $requestFields = array_merge($requestFields,$this->additionalParams);

        $cacheId = 'navigation_'.$this->startfolderId.'&langs='.serialize($sysLangs);
        $updateTimestamp = $this->_getYoungestTimestamp($this->startfolderId, $cacheId);
        $cacheId .= $updateTimestamp;
		$data = array();
        $filtersList = '';

        //Load data
        if (($data = Yii::app()->cache->get($cacheId)) === false) {
            if ($filters = $this->getFiltersList()) {
                $filtersList = implode(',', $filters);
            }

            $data = Yii::app()->repositoryApi->requestNavigation(
                array(
                    'rootfilter' => 'object.contentid=="'.$this->startfolderId.'"',
                    'sorting' => $this->sortby,
                    'navigation' => 'true',
                    'childfilter' => 'object.obj_type==10002 AND object.navhidden!=1',
                    'type' => 'php',
                ),
                CMap::mergeArray($this->getFiltersList(), $requestFields)
            );
            $data = unserialize($data);

            Yii::app()->cache->set($cacheId, $data, $this->getTreeCacheTime());
        }
        $this->data = isset($data[$this->startfolderId]) ? $data[$this->startfolderId] : array();
        
        // removed it for smaller debug log 
        //Yii::trace(print_r($this->data, true), 'navigation tree');

    }

    private function _getYoungestTimestamp($objectId, $objectCacheId)
    {
        $cacheId = 'youngest_' . $objectCacheId;
        if (($updateTimestamp = Yii::app()->cache->get($cacheId)) == false) {
            $updateTimestamp = Yii::app()->repositoryApi->requestNavigation(
                array(
                    "filter" => 'object.contentid=="' . $objectId . '"',
                    "type" => "youngest"
                ),
                array("updatetimestamp")
            );
            Yii::app()->cache->set($cacheId, $updateTimestamp, $this->getLatestUpdateTimestampCacheTime());
        }

        return $updateTimestamp;
    }

    /**
     * Load active path for current startfolderId and activeElement. If not exists request Gentics Content Connector API.
     *
     * @return array
     */
    private function _loadActivePath()
    {
        $cacheId = md5('activePath_' . NavigationTree::getUpdatetimestampByContentId($this->startfolderId) . '_' . $this->startfolderId . '_' . $this->activeElement);

        //Load data
        if (($data = Yii::app()->cache->get($cacheId)) === false) {
            if (!empty($this->activeElement)) {
                $data = Yii::app()->repositoryApi->requestActivePath(
                    array(
                        'root' => $this->startfolderId,
                        'filter' => 'object.contentid=="'.$this->activeElement.'"',
                        'type' => 'php',
                    )
                );

                $data = unserialize($data);
                unset($data['status']);
                $data = array_keys($data);
            } else {
                $data = array(0);
            }

            Yii::app()->cache->set($cacheId, $data, $this->getTreeCacheTime());
            Yii::trace('active path for element ' . $this->activeElement . ' putted to cache', 'test');
        } else {
            Yii::trace('active path for element ' . $this->activeElement . ' taken from cache', 'test');
        }

        $this->activePath = $data;
    }

    /**
     * returnes array with active path for given $startfolderId
     *
     * @param varchar $startfolderId contentId to check timestamp
     *
     * @return string|bool
     */
    public static function getUpdatetimestampByContentId($startfolderId)
    {
        $cacheId = 'navigation_' . $startfolderId;

        if (($data = Yii::app()->cache->get($cacheId)) === false) {
            return false;
        } else {
            return $data[(string)$startfolderId]['attributes']['updatetimestamp'];
        }
    }

    /**
     * Returnes branch with given id if exists
     *
     * @param array  $node      contains navigation tree
     * @param string $contentId contains id of content to search
     *
     * @return array|bool searched node
     */
    public static function extractBranch($node = array(), $contentId = '')
    {
        if ($node['contentid'] == $contentId) {
            return $node;
        } elseif (!empty($node['children'])) {
            foreach ($node['children'] as $childNode) {
                if ($searchedNode = self::extractBranch($childNode, $contentId)) {
                    return $searchedNode;
                }
            }
        }

        return false;
    }
}
